package com.persagy.ems.pojo.wz;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.persagy.core.annotation.Column;
import com.persagy.core.annotation.Dimension;
import com.persagy.core.annotation.Entity;
import com.persagy.core.annotation.Index;
import com.persagy.core.annotation.Table;
import com.persagy.core.enumeration.Schema;
import com.persagy.core.enumeration.EMSDimension;
import com.persagy.core.mvc.pojo.BusinessObject;
import java.util.Date;

@Dimension(dimension = EMSDimension.Building)
@Entity(name = "DayStaticRecord")
@Table(name = "t_day_static", comment = "天气数据统计", schema = Schema.EMS, indexes = {
		@Index(columns = { "c_city_Id", "c_day_time" }, unique = true),
		@Index(columns = { "c_day_time", "c_city_Id" }) })
public class DayStaticRecord extends BusinessObject {
	private static final long serialVersionUID = 8185957512341152181L;
	
	@JsonProperty("cityId")
	@Column(order = 2, name = "c_city_Id", length = 40, nullable = false, comment = "城市编码")
	private String cityId;
	@JsonProperty("dayTime")
	@Column(order = 3, name = "c_day_time", length = 0, nullable = false, comment = "日开始时间")
	private Date dayTime;
	@JsonProperty("updateTime")
	@Column(order = 3, name = "c_update_time", length = 0, nullable = false, comment = "数据保存时间")
	private Date updateTime;
	@Column(order = 4, name = "c_max_temp", length = 40, nullable = true, comment = "最高温度，单位c摄氏度或f华氏度")
	@JsonProperty("maxTemp")
	private Double maxTemp;
	@Column(order = 5, name = "c_min_temp", length = 40, nullable = true, comment = "最低温度，单位c摄氏度或f华氏度")
	@JsonProperty("minTemp")
	private Double minTemp;
	@Column(order = 7, name = "c_max_wind_speed", length = 40, nullable = true, comment = "最大风速，单位km/h")
	@JsonProperty("maxWindSpeed")
	private Double maxWindSpeed;
	@Column(order = 8, name = "c_min_wind_speed", length = 40, nullable = true, comment = "最小风速，单位km/h")
	@JsonProperty("minWindSpeed")
	private Double minWindSpeed;
	@Column(order = 9, name = "c_max_humidity", length = 40, nullable = true, comment = "相对湿度，0~100，单位百分比")
	@JsonProperty("maxHumidity")
	private Double maxHumidity;
	@Column(order = 10, name = "c_min_humidity", length = 40, nullable = true, comment = "相对湿度，0~100，单位百分比")
	@JsonProperty("minHumidity")
	private Double minHumidity;
	@Column(order = 11, name = "c_max_pressure", length = 40, nullable = true, comment = "气压，单位mb百帕")
	@JsonProperty("maxPressure")
	private Double maxPressure;
	@Column(order = 12, name = "c_min_pressure", length = 40, nullable = true, comment = "气压，单位mb百帕")
	@JsonProperty("minPressure")
	private Double minPressure;
	@Column(order = 13, name = "c_max_visibility", length = 40, nullable = true, comment = "能见度，单位为km公里")
	@JsonProperty("maxVisibility")
	private Double maxVisibility;
	@Column(order = 14, name = "c_min_visibility", length = 40, nullable = true, comment = "能见度，单位为km公里")
	@JsonProperty("minVisibility")
	private Double minVisibility;

	public String getCityId() {
		return cityId;
	}

	public void setCityId(String cityId) {
		this.cityId = cityId;
	}

	public Date getDayTime() {
		return dayTime;
	}

	public void setDayTime(Date dayTime) {
		this.dayTime = dayTime;
	}

	public Date getUpdateTime() {
		return updateTime;
	}

	public void setUpdateTime(Date updateTime) {
		this.updateTime = updateTime;
	}

	public Double getMaxTemp() {
		return maxTemp;
	}

	public void setMaxTemp(Double maxTemp) {
		this.maxTemp = maxTemp;
	}

	public Double getMinTemp() {
		return minTemp;
	}

	public void setMinTemp(Double minTemp) {
		this.minTemp = minTemp;
	}

	public Double getMaxWindSpeed() {
		return maxWindSpeed;
	}

	public void setMaxWindSpeed(Double maxWindSpeed) {
		this.maxWindSpeed = maxWindSpeed;
	}

	public Double getMinWindSpeed() {
		return minWindSpeed;
	}

	public void setMinWindSpeed(Double minWindSpeed) {
		this.minWindSpeed = minWindSpeed;
	}

	public Double getMaxHumidity() {
		return maxHumidity;
	}

	public void setMaxHumidity(Double maxHumidity) {
		this.maxHumidity = maxHumidity;
	}

	public Double getMinHumidity() {
		return minHumidity;
	}

	public void setMinHumidity(Double minHumidity) {
		this.minHumidity = minHumidity;
	}

	public Double getMaxPressure() {
		return maxPressure;
	}

	public void setMaxPressure(Double maxPressure) {
		this.maxPressure = maxPressure;
	}

	public Double getMinPressure() {
		return minPressure;
	}

	public void setMinPressure(Double minPressure) {
		this.minPressure = minPressure;
	}

	public Double getMaxVisibility() {
		return maxVisibility;
	}

	public void setMaxVisibility(Double maxVisibility) {
		this.maxVisibility = maxVisibility;
	}

	public Double getMinVisibility() {
		return minVisibility;
	}

	public void setMinVisibility(Double minVisibility) {
		this.minVisibility = minVisibility;
	}
}
